package mpi.eudico.client.annotator;

import java.awt.EventQueue;
import java.io.File;

import javax.swing.JPopupMenu;
import javax.swing.UIManager;

import mpi.eudico.client.annotator.turnsandscenemode.TaSFrame;
import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.client.annotator.util.SystemReporting;

public class SIMPLELAN {
    /** the major version value */
    public static int major = 1;

    /** the minor/bug-fix version value */
    public static int minor = 1;
    
    private final static String appName = "Simple-ELAN";
    /**
     * Private constructor
     */
	private SIMPLELAN() {

	}

	public static void main(final String[] args) {
		System.out.println("SIMPLE-ELAN Launch");
		System.setProperty("ELANApplicationMain", SIMPLELAN.class.getName());
		
		// make sure the directory for Elan data exists, could move
        try {
            if (System.getProperty("os.name").indexOf("Mac OS") > -1) {
            	File dataFolder = new File(Constants.ELAN_DATA_DIR);
            	if (!dataFolder.exists()) {
            		dataFolder.mkdir();
            	}
            	boolean screenBar = false;//default
            	Object val = Preferences.get("OS.Mac.useScreenMenuBar", null);
            	if (val instanceof Boolean) {
            		screenBar = ((Boolean) val).booleanValue();
            		System.setProperty("apple.laf.useScreenMenuBar", String.valueOf(screenBar));
            	}
            	// using the screen menu bar implies the default Mac OS L&F
            	if (!screenBar) {
            		val = Preferences.get("UseMacLF", null);
            		
            		if (val instanceof Boolean) {
            			boolean macLF = ((Boolean) val).booleanValue();
            			if (!macLF) {
                            try {
                                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                                JPopupMenu.setDefaultLightWeightPopupEnabled(false);//??
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
            			}
            		}
            	}
            	//System.setProperty("apple.awt.brushMetalLook", "true"); 
            	// media framework: if no framework specified, check the user's stored preference
            	if (System.getProperty("PreferredMediaFramework") == null) {
            		val = Preferences.get("Mac.PrefMediaFramework", null);
            		if (val instanceof String) {
            			System.setProperty("PreferredMediaFramework", (String) val);
            		}
            	}

            }// end mac initialization
            else if (SystemReporting.isWindows()) {// windows user preferred media framework
            	if (System.getProperty("PreferredMediaFramework") == null) {
            		Object val = Preferences.get("Windows.PrefMediaFramework", null);
            		if (val instanceof String) {
            			System.setProperty("PreferredMediaFramework", (String) val);
            		}
            	}

            	Object val = Preferences.get("UseWinLF", null);
            	if (val instanceof Boolean) {
            		if ((Boolean) val) {
                    	try {
                        	UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                        } catch (Exception ex) {
                        	ClientLogger.LOG.warning("Could not set the Look and Feel");
                        }
            		}
            	}
            }
            else if (SystemReporting.isLinux()) {// Linux user preferred media framework
            	if (System.getProperty("PreferredMediaFramework") == null) {
            		Object val = Preferences.get("Linux.PrefMediaFramework", null);
            		if (val instanceof String) {
            			System.setProperty("PreferredMediaFramework", (String) val);
            		}
            	}
            }
            
            File dataDir = new File(Constants.ELAN_DATA_DIR);

            if (!dataDir.exists()) {
                dataDir.mkdir();
            }

        } catch (Exception ex) {
            // catch any
        	ClientLogger.LOG.warning("Could not create ELAN's data directory: " + ex.getMessage());
        }
        
        //FrameManager.getInstance().setExitAllowed(true);
        // create the frame on the event dispatch thread
        
        EventQueue.invokeLater(new Runnable() {
                @Override
				public void run() {
                	TaSFrame slFrame = null;
                	if ((args != null) && (args.length > 0) && (args[0].length() != 0)) {
                		File argFile = new File(args[0]); 
                		//System.out.println("F " + argFile.getAbsolutePath());
                		//System.out.println("A " + argFile.isAbsolute());
                    	// HS July 2008: check if the argument (filepath to eaf) is a relative
                    	// path. If so let the jvm resolve it relative to the current directory 
                    	// (where ELAN is launched from)
                    	if (!argFile.isAbsolute()) { 
                    		//System.out.println("F " + argFile.getAbsolutePath());
                    		//FrameManager.getInstance().createFrame(argFile.getAbsolutePath());
                    		slFrame = new TaSFrame(argFile.getAbsolutePath());
                    		//FrameManager.getInstance().addFrame(slFrame);
                    	} else {
                    		//FrameManager.getInstance().createFrame(args[0]);
                    		slFrame = new TaSFrame(args[0]);
                    		//FrameManager.getInstance().addFrame(slFrame);
                    	}	                    	
                	} else {
                		slFrame = new TaSFrame();
                	}
                	
                	if (slFrame != null) {
                		slFrame.setVisible(true);
                		
                		if (System.getProperty("os.name").indexOf("Mac OS") > -1) {
                			try {
                            	Class<?> macHandler = null;
                            	try {
                            		macHandler = Class.forName("mpi.eudico.client.mac.MacAppHandler2");
                            	} catch (Throwable anyCause) {
                            		System.out.println("Loading Apple integration 2 failed");
                            		//anyCause.printStackTrace();
                            	}
                                Class<?> macList = Class.forName(
                                        "mpi.eudico.client.annotator.turnsandscenemode.TaSFrame$SimpleELANMacApplication");
                                Class<?> superList = macList;
                                if (macList.getInterfaces().length > 0) {
                                	superList = macList.getInterfaces()[0];
                                }
                                java.lang.reflect.Constructor<?> con = macHandler.getConstructor(superList);
                                con.newInstance(slFrame.new SimpleELANMacApplication());
                            } catch (Throwable ex) {
                                System.out.println("Could not load Mac application handler.");
                                //ex.printStackTrace();
                            }
                		}
                	}
                }
            });
           
	}
	
	/**
	 * @return the name of the application as shown to the users
	 */
	public static String getApplicationName() {
		return appName;
	}
	
    /**
     * Returns the current version information as a string.
     *
     * @return the current version
     */
    public static String getVersionString() {
        return major + "." + minor;
    }
    
}
