package mpi.eudico.client.annotator.commands;

import java.util.List;

import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.event.ACMEditEvent;
/**
 * Iterates over top level tiers and shifts annotations within specified interval
 * taking into account limits based on existing annotations.
 * 
 *  @see ShiftAnnotationsCommand
 */
public class ShiftAnnotationsLROfCommand extends ShiftAnnotationsCommand {
    
    
	public ShiftAnnotationsLROfCommand(String name) {
		super(name);
		availBT = 0;
		availET = Long.MAX_VALUE;
	}

    /**
     * <b>Note: </b>it is assumed the types and order of the arguments are
     * correct.
     *
     * @param receiver the Transcription
     * @param arguments the arguments: <ul><li>arg[0] = the begin of the time interval the
     *        annotations in which are  to be shifted (Long)</li> <li>arg[1] =
     *        the end time of that interval (Long)</li> <li>arg[2] = the shift
     *        value (Long)</li> </ul>
     */
	@Override
	public void execute(Object receiver, Object[] arguments) {
		transcription = (TranscriptionImpl) receiver;
		
        if (arguments != null) {
            bt = (Long) arguments[0];
            et = (Long) arguments[1];
            shiftValue = (Long) arguments[2];

            transcription.setNotifying(false);
            int numShifted = 0;
            
            List<TierImpl> rootTiers = transcription.getTopTiers();
            // calculate overall available space first 
            for (int i = 0; i < rootTiers.size(); i++) {
            	TierImpl tier = rootTiers.get(i);
            	
            	availBT = Math.max(availBT, getBoundaryBefore(tier, bt));
            	availET = Math.min(availET, getBoundaryAfter(tier, et));
            }            
            
            for (int i = 0; i < rootTiers.size(); i++) {
            	TierImpl tier = rootTiers.get(i);
            	
            	if (tier != null && tier.isTimeAlignable()) {
            		numShifted += shift(tier, shiftValue, bt, et);
            	}
            }
            
            if (numShifted > 0) {
            	transcription.handleModification(transcription, ACMEditEvent.CHANGE_ANNOTATIONS, 
            			transcription);
            }
            
            transcription.setNotifying(true);
        }
	}

    /**
     * Shift the annotations again.
     */
	@Override
	public void redo() {
		if (transcription != null) {
			transcription.setNotifying(false);
			int numShifted = 0;
			
            List<TierImpl> rootTiers = transcription.getTopTiers();
            for (int i = 0; i < rootTiers.size(); i++) {
            	TierImpl tier = rootTiers.get(i);
            	
            	if (tier != null && tier.isTimeAlignable()) {
            		numShifted += shift(tier, shiftValue, bt, et);
            	}
            }
            
            if (numShifted > 0) {
            	transcription.handleModification(transcription, ACMEditEvent.CHANGE_ANNOTATIONS, 
            			transcription);
            }
            
            transcription.setNotifying(true);
		}
	}

    /**
     * Shift the annotations back.
     */
	@Override
	public void undo() {
		if (transcription != null) {
			transcription.setNotifying(false);
			int numShifted = 0;
			
            List<TierImpl> rootTiers = transcription.getTopTiers();
            for (int i = 0; i < rootTiers.size(); i++) {
            	TierImpl tier = rootTiers.get(i);
            	
            	if (tier != null && tier.isTimeAlignable()) {
//            		numShifted += shift(tier, -shiftValue, (bt == 0 ? bt : (bt + shiftValue)),
//                            ((et == Long.MAX_VALUE) ? et : (et + shiftValue)));
            		numShifted += shift(tier, -shiftValue, availBT, availET);
            	}
            }
            
            if (numShifted > 0) {
            	transcription.handleModification(transcription, ACMEditEvent.CHANGE_ANNOTATIONS, 
            			transcription);
            }
            
            transcription.setNotifying(true);
		}
	}
    
    
}
