package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Shape;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.interlinear.IGTTierType;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDataModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTNodeRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTTierRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTViewerRenderInfo;
import mpi.eudico.client.annotator.util.ClientLogger;

/**
 *  A class for rendering a row of data (a group) into the specified graphics context.
 *  
 * @author Han Sloetjes
 */
public class IGTGroupRenderer implements ClientLogger {
	private final static boolean debugTierType = false;
	private final static boolean showAnnoBorders = true;
	private final static Color ANNO_BORDER_COLOR = Color.LIGHT_GRAY;
	
	/**
	 * Renders a row / group into the specified graphics context.
	 * Seems unused (renderRow2 is used instead)
	 */
	private void renderRowOldAndUnused(Graphics g2d, IGTDataModel model, IGTViewerRenderInfo vrInfo, int row,
			int xOff, int yOff, int width, int height) {
		// clear the rectangle for this row
		if (row % 2 == 0) {
			g2d.setColor(vrInfo.backgroundColor);
		} else {
			g2d.setColor(vrInfo.backgroundColor2);
		}
		g2d.fillRect(xOff, yOff, width, height);
		
		g2d.setColor(vrInfo.foregroundColor);
		g2d.drawLine(vrInfo.headerWidth - 1, yOff, vrInfo.headerWidth - 1, yOff + height);
		
		// start rendering tiers, tier labels, annotations
		int tempY = yOff;
		
		List<IGTTier> renderedTier = new ArrayList<IGTTier>();
		for (int i = 0; i < model.getRowCount(); i++) {
			IGTTier tier = model.getRowData(i);
			if (renderedTier.contains(tier)) {
				continue;
			}
			int tierHeight = vrInfo.getHeightForTier(g2d, tier.getTierName());
			int baseline = vrInfo.getBaselineForTier(g2d, tier.getTierName());
			
			if (tier.getType() == IGTTierType.WORD_LEVEL_ROOT /*|| tier.getType() == IGTTierType.SUBDIVISION*/) {
				int numBlocks = 1;
				@SuppressWarnings("null")
				List<Integer> wrapIndices = tier.getRenderInfo().getWrapIndices();
				if (wrapIndices != null) {
					numBlocks = wrapIndices.size() + 1;
				}
				// render the group of tiers in blocks with or without repeating the tier label
				IGTTier depTier;
				for (int j = 0; j < numBlocks; j++) {
					tempY += tierHeight; 
					g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - baseline); // position the baseline of the text correctly
					// render annotations on this tier 
					
					int minA = 0;
					int maxA = tier.getAnnotations().size() - 1;
					if (maxA < 0) {// no annotations
						continue; 
					}
					
					if (j > 0) { // false if wrapIndices == null, because then numBlocks == 1
						minA = wrapIndices.get(j - 1);
					} 
					if (j < numBlocks - 1) { // false if wrapIndices == null, same reason
						maxA = wrapIndices.get(j) - 1;
					}
					
					for (int a = minA; a <= maxA; a++) {
						IGTAnnotation igtAnn = tier.getAnnotations().get(a);
						// hier render annotation
						g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, 
								tempY - 2);// TODO position the text correctly vertically
						if (showAnnoBorders) {
							g2d.setColor(ANNO_BORDER_COLOR);
							g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tierHeight, 
									igtAnn.getRenderInfo().calcWidth, tierHeight);
							g2d.setColor(vrInfo.foregroundColor);
						}
					}

					int chMinX = tier.getAnnotations().get(minA).getRenderInfo().x;
					IGTNodeRenderInfo annRender = tier.getAnnotations().get(maxA).getRenderInfo();
					int chMaxX = annRender.x + annRender.calcWidth;
					
					for (int k = i + 1; k < model.getRowCount(); k++) {
						depTier = model.getRowData(k);
						int depTierHeight = depTier.getRenderInfo().height;
						if (depTier.hasAncestor(tier)) {
							tempY += vrInfo.vertLineMargin;
							tempY += depTierHeight;
							g2d.drawString(model.getShortTierNameForIndex(k), xOff, tempY - baseline);
							
							//render annotations on the dep. tier
							for (int a = 0; a < depTier.getAnnotations().size(); a++) {
								IGTAnnotation igtAnn = depTier.getAnnotations().get(a);
								annRender =igtAnn.getRenderInfo();
								if (annRender.x < chMinX) {
									continue;
								}
								if (annRender.x > chMaxX) {
									break;
								}
								
								g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + annRender.x, 
										tempY - 2);// TODO position the text correctly vertically
								if (showAnnoBorders) {
									g2d.setColor(ANNO_BORDER_COLOR);
									g2d.drawRect(xOff + vrInfo.headerWidth + annRender.x, tempY - depTierHeight, 
											annRender.calcWidth, depTierHeight);
									g2d.setColor(vrInfo.foregroundColor);
								}
							}
							
							renderedTier.add(depTier);
						} else {
							// if we assume that the depending tiers are following the "word" level tier immediately
							// we could break here
							break;
						}
					}
					
					tempY += vrInfo.vertLineMargin;
				}
				
				// update i such that the dependent children are skipped?? TODO support a tier order list and a list of visible tiers
			} else { // render root and association tier
				tempY += tierHeight; 
				g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2); // position the baseline of the text correctly
				if (tier.getRenderInfo().getNumLines() > 1) {
					if (tier.getAnnotations().size() >= 1) {
						IGTAnnotation igtAnn = tier.getAnnotations().get(0);
						List<String> lines = igtAnn.getRenderInfo().getWrappedLines();
						if (lines != null) {
							for (int a = 0; a < lines.size(); a++) {
								g2d.drawString(lines.get(a), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - baseline);
								
								if (showAnnoBorders) {
									g2d.setColor(ANNO_BORDER_COLOR);
									g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tierHeight, 
											igtAnn.getRenderInfo().calcWidth, tierHeight);
									g2d.setColor(vrInfo.foregroundColor);
								}
								
								if (a != lines.size() - 1) {
									tempY += vrInfo.vertLineMargin;
									tempY += tierHeight;
								}
							}
						} //else {//test code
//							g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
//							if (showAnnoBorders) {
//								g2d.setColor(ANNO_BORDER_COLOR);
//								g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - vrInfo.tierHeight, 
//										igtAnn.getRenderInfo().calcWidth, vrInfo.tierHeight);
//								g2d.setColor(vrInfo.foregroundColor);
//							}
//						}					
					} 
				} else {
					if (tier.getAnnotations().size() >= 1) {
						IGTAnnotation igtAnn = tier.getAnnotations().get(0);
						g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - baseline);
						if (showAnnoBorders) {
							g2d.setColor(ANNO_BORDER_COLOR);
							g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tierHeight, 
									igtAnn.getRenderInfo().calcWidth, tierHeight);
							g2d.setColor(vrInfo.foregroundColor);
						}
					}
				}
			}
			renderedTier.add(tier);
			// render annotations?
			tempY += vrInfo.vertLineMargin;
		}
	}
		
	/**
	 * Renders a row / group into the specified graphics context.
	 * <p>
	 * It is assumed that the background has been cleared to the desired background colour.
	 * <p>
	 * See also IGTSuggestionRenderer.renderRowData(...).
	 * 
	 * @param suggestionSourceAnn To determine which annotation should be highlighted 
	 * @param selectedAnn 
	 */
	public static void renderRow2(Graphics g, IGTDataModel model, IGTViewerRenderInfo vrInfo,
			int height, IGTAnnotation selectedAnn, IGTAnnotation suggestionSourceAnn) {
		
		boolean showTierLabels = true;
		boolean isSuggestion = model instanceof IGTSuggestionModel;
		if (isSuggestion) {
			showTierLabels = ((IGTSuggestionModel)model).showTierLabels();
		}
		
		g.setColor(vrInfo.foregroundColor);
		
		int labelWidth;
		if (showTierLabels) {
			labelWidth = vrInfo.headerWidth;
			g.drawLine(labelWidth - 1, 0, labelWidth - 1, height);
		} else {
			labelWidth = 0;
		}
		
		// start rendering tiers, tier labels, annotations
		int defFontBaseline = g.getFontMetrics(vrInfo.defaultFont).getDescent();
				
		List<IGTTier> renderedTier = new ArrayList<IGTTier>();
		final int rowCount = model.getRowCount();
		
		for (int i = 0; i < rowCount; i++) {
			IGTTier tier = model.getRowData(i);
			if (renderedTier.contains(tier)) {
				continue;
			}
			if (isSuggestion && tier.getType() == IGTTierType.ROOT) {
				continue;
			}
			
			final IGTTierRenderInfo tierRenderInfo = tier.getRenderInfo();
			final ArrayList<IGTAnnotation> annotations = tier.getAnnotations();
			final int numAnnos = annotations.size();
			final int depth = 10 * getDepth(tier);
			final String tierType = debugTierType ? tierTypeTag(tier) : "";
			
			if (tier.isInWordLevelBlock()) {
				// tier labels
				if (showTierLabels) {
					g.setFont(vrInfo.defaultFont);
					g.setColor(vrInfo.getColorForTier(tier));
					// The user clip is apparently used by JTable to clip the editor.
					Shape oldClip = g.getClip();
					g.clipRect(0, 0, labelWidth - 1, height);
					
					final List<Integer> yPositions = tierRenderInfo.getYPositions();
	
					// There may be block-wrapping in word-level blocks.
					if (yPositions != null) {
						final int numYpositions = yPositions.size();
	
						for (int j = 0; j < numYpositions; j++) {
							int botY = yPositions.get(j) + tierRenderInfo.height;
							// TODO use the font from the tier render info 
							//g2d.setFont(tier.getRenderInfo().tierFont);
							g.drawString(model.getShortTierNameForIndex(i) + tierType, depth, botY - defFontBaseline);
							if (LOG.isLoggable(Level.FINER)) {
								LOG.finer(String.format("string 1T: '%s'",
										model.getShortTierNameForIndex(i)));
							}
						}
					} else {
						int botY = tierRenderInfo.y + tierRenderInfo.height;
						// TODO use the font from the tier render info 
						//g2d.setFont(tier.getRenderInfo().tierFont);
						g.drawString(model.getShortTierNameForIndex(i) + tierType, depth, botY - defFontBaseline);
						if (LOG.isLoggable(Level.FINER)) {
							LOG.finer(String.format("string 2T: '%s'",
									model.getShortTierNameForIndex(i)));
						}
					}
					
					g.setClip(oldClip);
				}
				
				// annotations
				g.setFont(vrInfo.getFontForTier(tier.getTierName()));
				g.setColor(vrInfo.foregroundColor);
				int baseline = vrInfo.getBaselineForTier(g, tier.getTierName());

				for (int j = 0; j < numAnnos; j++) {
					IGTAnnotation igtAnn = annotations.get(j);
					IGTNodeRenderInfo annRender = igtAnn.getRenderInfo();
					int botY = annRender.y + annRender.height;
					
//					if (debugDrawString) System.out.printf("string 3A: '%s'\n", igtAnn.getTextValue());
					drawBox(igtAnn == selectedAnn, igtAnn == suggestionSourceAnn,
							labelWidth + annRender.x, annRender.y,
							annRender.calcWidth, annRender.height,
							g, igtAnn.getTextValue(), botY - baseline);
				}
			} else {
				// other types of tiers: a single long annotation;
				// maybe it's text lines are internally line-wrapped.
				int botY = tierRenderInfo.y + tierRenderInfo.height;	// bottom of whole tier
				
				// Tier labels
				if (showTierLabels) {
					Shape oldClip = g.getClip();
					g.clipRect(0, 0, labelWidth - 1, height);
					g.setFont(vrInfo.defaultFont);
					g.setColor(vrInfo.getColorForTier(tier));
					g.drawString(model.getShortTierNameForIndex(i) + tierType, depth, botY - defFontBaseline); // position the baseline of the text correctly
					if (LOG.isLoggable(Level.FINER)) {
						LOG.finer(String.format("string 4T: '%s'",
								model.getShortTierNameForIndex(i)));
					}
					g.setClip(oldClip);
				}
				
				// Annotation
				if (tierRenderInfo.getNumLines() > 1) {
					if (numAnnos >= 1) {
						IGTAnnotation igtAnn = annotations.get(0);
						final IGTNodeRenderInfo annRender = igtAnn.getRenderInfo();
						List<String> lines = annRender.getWrappedLines();
						
						if (lines != null) {
							g.setFont(vrInfo.getFontForTier(tier.getTierName()));
							g.setColor(vrInfo.foregroundColor);
							int baseline = vrInfo.getBaselineForTier(g, tier.getTierName());

							final int numLines = lines.size();
							for (int a = 0; a < numLines; a++) {
								if (LOG.isLoggable(Level.FINER)) {
									LOG.finer(String.format("string 5L: '%s'",
											lines.get(a)));
								}
								drawBox(igtAnn == selectedAnn, igtAnn == suggestionSourceAnn,
										labelWidth + annRender.x, botY - tierRenderInfo.height,
										annRender.calcWidth, tierRenderInfo.height,
										g, lines.get(a), botY - baseline);
								
								botY += vrInfo.vertLineMargin;
								botY += tierRenderInfo.height;
							}
						} 				
					} 
				} else {
					if (numAnnos >= 1) {
						g.setFont(vrInfo.getFontForTier(tier.getTierName()));
						g.setColor(vrInfo.foregroundColor);
						int baseline = vrInfo.getBaselineForTier(g, tier.getTierName());

						IGTAnnotation igtAnn = annotations.get(0);
						final IGTNodeRenderInfo annRenderInfo = igtAnn.getRenderInfo();
						if (LOG.isLoggable(Level.FINER)) {
							LOG.finer(String.format("string 6A: '%s'",
									igtAnn.getTextValue()));
						}
						drawBox(igtAnn == selectedAnn, igtAnn == suggestionSourceAnn,
								labelWidth + annRenderInfo.x, botY - tierRenderInfo.height,
								annRenderInfo.calcWidth, tierRenderInfo.height,
								g, igtAnn.getTextValue(), botY - baseline);
					}
				}
			}
			
			renderedTier.add(tier);
		}
	}

	private static void drawBox(boolean isSelected, boolean isSugg, int x, int y, int w, int h,
			Graphics g, String text, int textY) {
		Color saveColor = g.getColor();

		if (isSugg) {
			// isSugg overrides isSelected, because visually the latter drowns the former.
			g.setColor(Color.GREEN);
			g.drawRect(x, y, w - 1, h - 1);
			g.setColor(saveColor);
		} else if (isSelected) {
//			g.setColor(Color.GREEN);
//			g.fillRect(x, y, w, h);
			g.setColor(Constants.ACTIVEANNOTATIONCOLOR);
			g.drawRect(x, y, w - 1, h -1);
			g.setColor(saveColor);
		} else if (showAnnoBorders) {
			g.setColor(ANNO_BORDER_COLOR);
			g.drawRect(x, y, w - 1, h - 1);
			g.setColor(saveColor);
		}
		
		g.drawString(text, x, textY);
	}
	
	private static int getDepth(IGTTier tier) {
		if (tier.isSpecial()) {
			return 0;
		}
		
		int depth = -1;
		
		while (tier != null) {
			depth++;
			tier = tier.getParentTier();
		}
		
		return depth;
	}

	/**
	 * For debugging: make the sort of tier clear.
	 * 
	 * @param tier
	 * @return a string indicating the type
	 */
	private static String tierTypeTag(IGTTier tier) {
		if (showAnnoBorders) {
			String wlb = tier.isInWordLevelBlock() ? " inWLB" 
					                               : "";
			return " -- " + tier.getType().toString() + wlb;
		} else {
			return "";
		}
	}
}
