package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

import mpi.eudico.client.annotator.interlinear.IGTTierType;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDataModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionViewerModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTBlockRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTNodeRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTSuggestionRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTTierRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTViewerRenderInfo;

/**
 * A renderer for suggestion sets. The rendering is similar to that of igt groups,
 * but adds numbering of the sets and horizontal or vertical orientation of the display.  
 * 
 * @author Han Sloetjes
 */
public class IGTSuggestionRenderer {
	
	private static boolean debug = false;

	/**
	 * Returns the width of what was rendered, including a margin
	 */
	public static int renderOneSuggestion(Graphics g2d, 
			int sugIndex, boolean includeTierLabels,
			IGTViewerRenderInfo vrInfo,
			IGTSuggestionViewerModel svModel) {
		
		IGTSuggestionModel sugModel = svModel.getRowData(sugIndex);
		IGTBlockRenderInfo sugModelRenderInfo = sugModel.getRenderInfo();
		IGTSuggestionRenderInfo svModelRenderInfo = svModel.getRenderInfo();
		int height = sugModel.getRenderInfo().height;

		// draw header
		String slabel = "S" + (sugIndex + 1);
		String label = sugModel.getSuggestionSet().getLabel();
		if (label != null) {
			slabel = slabel + " " + label;
		}
		
		g2d.setColor(svModelRenderInfo.headerBackGround);
		g2d.fillRect(0, 0, sugModelRenderInfo.width, svModelRenderInfo.headerHeight);
		g2d.setColor(Color.BLACK);
		g2d.drawRect(0, 0, sugModelRenderInfo.width - 1, svModelRenderInfo.headerHeight);
		g2d.drawString(slabel, 2 + (includeTierLabels ? vrInfo.headerWidth : 0),
				       svModelRenderInfo.headerHeight - 2);
		
		// draw block background + line around it
		final int blockY = svModelRenderInfo.headerHeight + 1;
		g2d.setColor(svModelRenderInfo.blockBackGround);
		g2d.fillRect(0, blockY,
				sugModelRenderInfo.width, sugModelRenderInfo.height - blockY);
		g2d.setColor(Color.LIGHT_GRAY);
		g2d.drawRect(0, blockY, 
				sugModelRenderInfo.width - 1, sugModelRenderInfo.height - blockY - 1);

		IGTGroupRenderer.renderRow2(g2d, sugModel, vrInfo, height, null, null);
		// Note: includeTierLabels is missing from this call, but
		// the fact that sugModel is a IGTSuggestionModel is (ab)used.
		// As long as svModelRenderInfo.headerHeight is the same height as
		// the omitted ROOT tier, layout looks ok. This is arranged in
		// SuggestionComponent.calculateLocationAndSize(...) by manipulating
		// the initial Y position and thereby shifting all the tiers.
		
		int xOff = sugModelRenderInfo.width;
		xOff += svModelRenderInfo.suggestionMargin;
		
		return xOff;
	}
}
