package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SpringLayout;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.Zoomable;
import mpi.eudico.client.annotator.gui.FontSizePanel;
import mpi.eudico.client.annotator.interlinear.IGTTierType;
import mpi.eudico.client.annotator.interlinear.edit.config.AnalyzerConfig;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDefaultModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.layout.InterlinearizationManager;
import mpi.eudico.client.annotator.util.AnnotationCoreComparator;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.mediacontrol.ControllerEvent;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AlignableAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.event.ACMEditEvent;
import mpi.eudico.server.corpora.event.ACMEditListener;
import nl.mpi.lexan.analyzers.helpers.Position;

/**
 * The  InterlinearEditor contains a scrollbar, an IGTViewer controlled by that scrollbar,
 * and a button to start the Interlinearization process.
 * <pre>
 *   +----------------------------------------------------------------+
 *   |                                                                |
 *   |                 Button(s)                                      |
 *   |+-----------------------------------------------------------++-+|
 *   || IGTViewer                                                 ||s||
 *   || ....                                                      ||c||
 *   ||                                                           ||r||
 *   ||                                                           ||o||
 *   ||                                                           ||l||
 *   ||                                                           ||l||
 *   |+-----------------------------------------------------------++-+|
 *   +----------------------------------------------------------------+
 * </pre>
 */
@SuppressWarnings("serial")
public class InterlinearEditor extends AbstractViewer 
implements ComponentListener, ActionListener, ACMEditListener {
	private InterlinearizationManager manager;
	private TranscriptionImpl transcription = null;
	
    private Font font;
    private int bpHeight = 40;
    
    private JPanel buttonPanel;
    private JButton interButton;
    private JCheckBox recursiveCB;
    private FontSizePanel fontSizePanel;
    private JPanel playerControlPanel;
    private JSplitPane buttonSplitPane;
	private SpringLayout buttonLayout;
    
    private AnalyzerConfig activeConfig = null;
    private IGTViewer igtViewer = null;
    private IGTViewerModelImpl viewerModel = null;
	
	public InterlinearEditor(InterlinearizationManager manager) {
		super();
		this.manager = manager;
		this.transcription = (TranscriptionImpl) manager.getTranscription();
		initViewer();
	}
	
	public void configsChanged() {
		paintBuffer();
	}
	
	public void setActiveConfiguration(AnalyzerConfig ac) {
		activeConfig = ac;
	}
	
	public AnalyzerConfig getActiveConfiguration() {
		return activeConfig;
	}
	
	/**
	 * Returns the component that implements the Zoomable interface and will
	 * receive calls from key events.
	 * 
	 * @return the zoomable component, the font size panel
	 */
	public Zoomable getZoomable() {
		return fontSizePanel;
	}
	
	private void initViewer() {
		setOpaque(true);
        font = Constants.DEFAULTFONT;
        setFont(font);
        //metrics = getFontMetrics(font);
		setBackground(Color.WHITE);
		buttonLayout = new SpringLayout();
		buttonPanel = new JPanel(buttonLayout);
		playerControlPanel = new JPanel();
		buttonSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		buttonSplitPane.setOneTouchExpandable(false);
		buttonSplitPane.setLeftComponent(playerControlPanel);
		buttonSplitPane.setRightComponent(buttonPanel);
		buttonSplitPane.setContinuousLayout(true);
		buttonSplitPane.setBorder(null);
		//buttonSplitPane.setDividerSize(6);
		
		add(buttonSplitPane);
		interButton = new JButton();
		interButton.addActionListener(this);
		buttonPanel.add(interButton);
		recursiveCB = new JCheckBox();
		buttonPanel.add(recursiveCB);
		recursiveCB.addActionListener(this);
		//	
		viewerModel = new IGTViewerModelImpl();
		initModel();
		igtViewer = new IGTViewer(viewerModel, manager.getTextAnalyzerContext(), this);
		igtViewer.readGUIPreferences(transcription);
		add(igtViewer);
		//
		fontSizePanel = new FontSizePanel(igtViewer);
		buttonPanel.add(fontSizePanel);
		// layout buttonPanel
		buttonLayout.putConstraint(SpringLayout.WEST, interButton, 6, 
				SpringLayout.WEST, buttonPanel);
		buttonLayout.putConstraint(SpringLayout.VERTICAL_CENTER, interButton, 0, 
				SpringLayout.VERTICAL_CENTER, buttonPanel);
		buttonLayout.putConstraint(SpringLayout.WEST, recursiveCB, 6, 
				SpringLayout.EAST, interButton);
		buttonLayout.putConstraint(SpringLayout.VERTICAL_CENTER, recursiveCB, 0, 
				SpringLayout.VERTICAL_CENTER, buttonPanel);
		buttonLayout.putConstraint(SpringLayout.EAST, fontSizePanel, -3, 
				SpringLayout.EAST, buttonPanel);
		buttonLayout.putConstraint(SpringLayout.VERTICAL_CENTER, fontSizePanel, 0, 
				SpringLayout.VERTICAL_CENTER, buttonPanel);		
		//
		bpHeight = fontSizePanel.getPreferredSize().height + 4;
		buttonSplitPane.setDividerLocation(igtViewer.getHeaderWidth() - buttonSplitPane.getDividerSize() / 2);
		setPreferredFontAndColorSettings();
		addComponentListener(this);
		playerControlPanel.addComponentListener(this);
		updateLocale();
		
		paintBuffer();
	}
	
	/**
	 * Sets the preferred font and color setting for the tiers
	 *                         							
	 */
	private void setPreferredFontAndColorSettings(){			
		// preferred tier colors
		Map<String, Color> tierColorMap = new HashMap<String, Color>();
		Map<String, Color> colors = Preferences.getMapOfColor("TierColors", transcription);
		if (colors != null) {
			for (Map.Entry<String, Color> e : colors.entrySet()) {
				String name = e.getKey();
				Color col = e.getValue();
				Tier t = transcription.getTierWithId(name);
				if (t != null) {					
					tierColorMap.put(name, col);
				}
			}
		}
		
		igtViewer.setTierColorMap(tierColorMap);
		
		//preferred fonts
		Map<String, Font> fo = Preferences.getMapOfFont("TierFonts", transcription);
		if (fo != null) {
			igtViewer.setTierFontMap(fo);
		}
		
		//preferred font size
		Integer size = Preferences.getInt("InterlinearizationEditor.FontSize", transcription);
		if (size != null) {
			fontSizePanel.setFontSize(size);
		}
	}
	
	private void initModel() {		
        List<String> hiddenTiers = Preferences.getListOfString("InterlinearEditor.HiddenTiers", transcription);
        if (hiddenTiers == null) {
        	// try the tier set of the timeline viewer
        	hiddenTiers = Preferences.getListOfString("MultiTierViewer.HiddenTiers", transcription);
        	if (hiddenTiers == null) {
        		hiddenTiers = new ArrayList<String>();
        	}
        }
        viewerModel.setHiddenTiers(hiddenTiers);
        // read preferences for special tiers
        List<IGTTierType> hiddenSpecialTiers = new ArrayList<IGTTierType>(3);
        Boolean showTCPref = Preferences.getBool("InterlinearEditor.ShowTimeCode", transcription);
        if (showTCPref != null && !showTCPref) {
        	hiddenSpecialTiers.add(IGTTierType.TIME_CODE);
        	viewerModel.setSpecialTierVisibility(IGTTierType.TIME_CODE, showTCPref);// default is true, only set when false
        }
		
        Boolean showSpeakPref = Preferences.getBool("InterlinearEditor.ShowSpeaker", transcription);
        if (showSpeakPref != null && !showSpeakPref) {
        	hiddenSpecialTiers.add(IGTTierType.SPEAKER_LABEL);
        	viewerModel.setSpecialTierVisibility(IGTTierType.SPEAKER_LABEL, showSpeakPref);// default is true, only set when false
        }
		// get all annotations from all (toplevel) tiers, minus the hidden tiers
		List<AlignableAnnotation> allAnns = new ArrayList<AlignableAnnotation>();
		
		for (TierImpl t : transcription.getTiers()) {
			if (!t.hasParentTier() && !hiddenTiers.contains(t.getName())) {
				allAnns.addAll(t.getAlignableAnnotations());
			}
		}
		
		AnnotationCoreComparator comparator = new AnnotationCoreComparator();
		Collections.sort(allAnns, comparator);
		
		for (AlignableAnnotation aa : allAnns) {
			IGTDefaultModel rowModel = new IGTDefaultModel(aa, hiddenTiers, hiddenSpecialTiers);
			viewerModel.addRow(rowModel);
		}
	}
	
//	@Override
//	protected void paintComponent(Graphics g) {
//		super.paintComponent(g);
//	}

	@Override
	public void controllerUpdate(ControllerEvent event) {
		// stub

	}
	
	/**
	 * This is the time to store preferences...
	 */
	@Override
	public void isClosing() {
		Preferences.set("InterlinearizationEditor.FontSize", igtViewer.getFontSize(), 
				transcription, false, false);
		
		igtViewer.isClosing(transcription);
	}
	
	private void paintBuffer() {
        if ((getWidth() <= 0) || (getHeight() <= 0)) {
            return;
        }
        
        repaint();
	}
	
	/**
	 * For now, at any edit event, repaint
	 * @param e
	 */
	@Override
	public void ACMEdited(ACMEditEvent e) {
		// check the event type, and how to respond to each relevant event
		final int rowCount = viewerModel.getRowCount();
		switch (e.getOperation()) {
		case ACMEditEvent.ADD_ANNOTATION_HERE:
			
            if (e.getInvalidatedObject() instanceof TierImpl &&
                    e.getModification() instanceof AbstractAnnotation) {
            	AbstractAnnotation newAnn = (AbstractAnnotation) e.getModification();
                TierImpl invTier = (TierImpl) e.getInvalidatedObject();
                assert(invTier == newAnn.getTier());

                IGTDefaultModel rowModel = findAnnotationRow(newAnn, invTier);
                if (rowModel != null) {
    				// if this succeeds this generates an event 
    				rowModel.annotationAdded(newAnn);
                }
            }
            
			break;
		case ACMEditEvent.ADD_ANNOTATION_AFTER:
			
            if (e.getInvalidatedObject() instanceof TierImpl &&
                    e.getModification() instanceof AbstractAnnotation) {
            	AbstractAnnotation newAnn = (AbstractAnnotation) e.getModification();
                TierImpl invTier = (TierImpl) e.getInvalidatedObject();
                assert(invTier == newAnn.getTier());
                AbstractAnnotation annBefore = (AbstractAnnotation) invTier.getAnnotationBefore(newAnn);
                if (annBefore != null && annBefore.getParentAnnotation() == newAnn.getParentAnnotation()) {
	
	                IGTDefaultModel rowModel = findAnnotationRow(newAnn, invTier);
	                if (rowModel != null) {
        				// if this succeeds this generates an event 
        				rowModel.annotationAddedAfter(newAnn, annBefore);
	                }
                }// else warn??
            }
            
			break;
		case ACMEditEvent.ADD_ANNOTATION_BEFORE:

            if (e.getInvalidatedObject() instanceof TierImpl &&
                    e.getModification() instanceof AbstractAnnotation) {
            	AbstractAnnotation newAnn = (AbstractAnnotation) e.getModification();
                TierImpl invTier = (TierImpl) e.getInvalidatedObject();
                assert(invTier == newAnn.getTier());
                AbstractAnnotation annAfter = (AbstractAnnotation) invTier.getAnnotationAfter(newAnn);
                if (annAfter != null && annAfter.getParentAnnotation() == newAnn.getParentAnnotation()) {
	                
	                IGTDefaultModel rowModel = findAnnotationRow(newAnn, invTier);
	
	                if (rowModel != null) {
						// if this succeeds this generates an event 	
						rowModel.annotationAddedBefore(newAnn, annAfter);          	
	                }
                } // else warn??
            }
            
			break;
        case ACMEditEvent.CHANGE_ANNOTATION_VALUE:
        case ACMEditEvent.CHANGE_ANNOTATION_TIME:
        	// modification object is null
            if (e.getInvalidatedObject() instanceof AbstractAnnotation) {
            	AbstractAnnotation a = (AbstractAnnotation) e.getInvalidatedObject();
                
                TierImpl invTier = (TierImpl) a.getTier();
            	
                IGTDefaultModel rowModel = findAnnotationRow(a, invTier);
                if (rowModel != null) {
    				// if this succeeds this generates an event 
    				rowModel.annotationValueChanged(a);
                }
            }
            
            break;
            
        case ACMEditEvent.REMOVE_ANNOTATION:
        	// currently the transcription is the only information available in the event
        	// the deleted annotation needs to be accessible as well
            if (e.getInvalidatedObject() instanceof Transcription) {
                //System.out.println("Invalidated object: " + e.getInvalidatedObject());
            	
            	if (e.getModification() instanceof AbstractAnnotation) {
            		AbstractAnnotation remAnnotation = (AbstractAnnotation) e.getModification();
                    
                    TierImpl invTier = (TierImpl) remAnnotation.getTier();
                	
                    IGTDefaultModel rowModel = findAnnotationRow(remAnnotation, invTier);
                    if (rowModel != null) {
        				// if this succeeds this generates an event 
        				rowModel.annotationRemoved(remAnnotation);
                    }
            	} else {
            		// The case where getModification() == null.
            		// We must assume that anything is now potentially changed.
            		// Loop over ALL annotations in out model, and if any is markedDeleted,
            		// remove the IGTAnnotation for it.
            		// (Note that this case happens when applying suggestions but somehow
            		// the overlapped removed annotations are not yet marked deleted anyway...)
                    for (int i = 0; i < rowCount; i++) {
                    	IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                    	
                    	int tierRowCount = rowModel.getRowCount();
                    	for (int j = 0; j < tierRowCount; j++) {
                    		IGTTier tierRowModel = rowModel.getRowData(j);
                    		ArrayList<IGTAnnotation> anns = tierRowModel.getAnnotations();
                    		
                    		// Loop backwards because we remove elements from this ArrayList.
                    		for (int k = anns.size() - 1; k >= 0; k--) {
                    			IGTAnnotation igtAnn = anns.get(k);
                    			final AbstractAnnotation annotation = igtAnn.getAnnotation();
								//if (annotation != null) {
								//	System.out.printf("Ann: %s deleted: %s\n", annotation.getValue(), annotation.isMarkedDeleted());
								//}
								if (annotation != null && annotation.isMarkedDeleted()) {
                    				rowModel.annotationRemoved(annotation);
                    				tierRowCount = rowModel.getRowCount(); // this value might change if child tiers get empty???
                    			}
                    		}
                    	}             	
                     }
                    igtViewer.recalculateAllRows(true);
            	}
            }

            break;
            
        case ACMEditEvent.CHANGE_ANNOTATIONS:
        	// any number of annotations might have been changed. Assume a change in the current block only for now.
        	
        	eventChangeAnnotations();        	
        	break;
        	
        case ACMEditEvent.REMOVE_TIER:
        	Object obj  = e.getModification();
			if(obj instanceof TierImpl){
				TierImpl invTier = (TierImpl) obj;
                TierImpl rootTier = invTier.getRootTier();
                String rootTierName = rootTier.getName();
                
                for (int i = 0; i < rowCount; i++) {
                	IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                	IGTTier igtTier = rowModel.getRootRow();
                	
                 	if (igtTier.getTierName().equals(rootTierName)) {
                 		//igtTier.removeChildTier(invTier.getName());
                 		rowModel.tierRemoved(invTier.getName());
                 		if(invTier == rootTier){
                 			//viewerModel.removeRowData(rowModel);
                 			viewerModel.removeRow(i);
                 			i--;	// counteract the i++
                 		}
                 	}
                 }
                igtViewer.recalculateAllRows(true);
			}
			break;	
			
        case ACMEditEvent.ADD_TIER:
        	obj  = e.getModification();
			if (obj instanceof TierImpl) {
				TierImpl invTier = (TierImpl) obj;
                if (invTier.hasParentTier()) {
                	// A Child tier has been added
                	TierImpl rootTier = invTier.getRootTier();
                    String rootTierName = rootTier.getName();
                    
                    for (int i = 0; i < rowCount; i++) {
                    	IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                    	IGTTier igtTier = rowModel.getRootRow();
                    	
                     	if (igtTier.getTierName().equals(rootTierName)) {
                     		//igtTier.removeChildTier(invTier.getName());
                     		rowModel.tierAdded(invTier, viewerModel.getHiddenTiers());
                     	}
                     }
                    igtViewer.recalculateAllRows(true);
                } else {
                	// A root tier has been added
                	if (viewerModel.getHiddenTiers().contains(invTier.getName())) {
                		break;
                	}
                	List<AlignableAnnotation> allAnns = new ArrayList<AlignableAnnotation>();
                	allAnns.addAll(invTier.getAlignableAnnotations());
                	
            		int nextIndex = 0;
            		while (allAnns.size() > 0) {
            			AlignableAnnotation aa = allAnns.get(0);
            			IGTDefaultModel newRowModel = new IGTDefaultModel(aa, viewerModel.getHiddenTiers());
            			
            			if(nextIndex >= rowCount-1){
            				viewerModel.addRow(newRowModel);	
            				nextIndex++;
            			} else {
            				for (int i = nextIndex; i < rowCount; i++) {
            					IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                            	long beginTime = rowModel.getBeginTime();
                             	
                            	// begin time
                                if (aa.getBeginTimeBoundary() < beginTime) {
                                	viewerModel.insertRow(newRowModel, i);
                                	nextIndex = i+1;
                                	break;
                                } else if (aa.getBeginTimeBoundary() > beginTime) {
                                   continue;
                                }
                                
                                // begin time equal, compare end time
                                if (aa.getEndTimeBoundary() < rowModel.getEndTime()) {
                                	viewerModel.insertRow(newRowModel, i);
                                	nextIndex = i+1;
                                	break;
                                } else if (aa.getEndTimeBoundary() > rowModel.getEndTime()) {
                                	continue;
                                }
                             }
            			}
            		}
            		if (allAnns.size() > 0) {
                    	igtViewer.recalculateAllRows(true);
                    }
                }
			}
			break;
			
		default:
				// do nothing
		}
		paintBuffer();
	}

	/**
	 * @param changedAnn
	 * @param changedTier
	 * @param annAfter
	 */
	private IGTDefaultModel findAnnotationRow(AbstractAnnotation changedAnn,
			TierImpl changedTier) {
        long bt = changedAnn.getBeginTimeBoundary();
        long et = changedAnn.getEndTimeBoundary();
        
        TierImpl rootTier = changedTier.getRootTier();
        String rootTierName = rootTier.getName();

        final int rowCount = viewerModel.getRowCount();
		for (int i = 0; i < rowCount; i++) {
			IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
			IGTTier igtTier = rowModel.getRootRow();
			
			if (igtTier.getTierName().equals(rootTierName)) {
				List<IGTAnnotation> annos = igtTier.getAnnotations();
				if (annos != null && annos.size() > 0) {
					IGTAnnotation igtAnn = annos.get(0);
					
					if (igtAnn.getAnnotation().getBeginTimeBoundary() <= bt && 
							igtAnn.getAnnotation().getEndTimeBoundary() >= et) {
						return rowModel;
					}
				}
			}
		}
		
		return null;
	}

	/**
	 * Handle "unspecified annotations" being changed.
	 * <p>
	 * Currently, if the editor is active, limit the updates it its IGTGroup.
	 * This assumes that any modifications must be initiated from the editor.
	 * <p>
	 * Otherwise, just rebuild all IGTGroups.
	 */
	private void eventChangeAnnotations() {
		int origditRow = viewerModel.getEditingRow();

		if (origditRow >= 0) {
			int row = origditRow;
			IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(row);
			// Check if this works:
			// determine begin and end time of this row, and create new rows for
			// all annotations that now happen to be inside that interval on the root tier.
			long beginTime = rowModel.getBeginTime(); // this is the original time
			long endTime = rowModel.getEndTime();     // this is the original time
			String tierName = rowModel.getRootRow().getTierName();
			TierImpl tier = transcription.getTierWithId(tierName);
			
			viewerModel.removeRow(row);
			// At this stage we likely don't have to preserve any listeners that the
			// replaced rows had. All those that exist get (re)created when we make the
			// replacements.
			
			for (AlignableAnnotation anno : tier.getAlignableAnnotations()) {
				if (anno.getEndTimeBoundary() <= beginTime) {
					continue;
				}
				if (anno.getBeginTimeBoundary() >= endTime) {
					break;
				}

				IGTDefaultModel replacementRowModel = 
						new IGTDefaultModel(anno,
								            viewerModel.getHiddenTiers());
				
				viewerModel.insertRow(replacementRowModel, row);
				
				row++;
			}
			
			// The alternative, reloading the whole model, seems too drastic,
			// certainly when we're in the middle of editing some row and we won't know
			// if this row will remain in the same position...
			// Also it seems that sometimes we don't even see all changes if we reload the whole model!
			// Maybe we get this notification before the children are added to the root annotation?
		} else {
			// reload the whole model
			//Temp
			while (viewerModel.getRowCount() > 0) {
				viewerModel.removeRow(viewerModel.getRowCount() - 1);
		    }
			initModel();
		}
		igtViewer.recalculateAllRows(true);  // Generates a viewChanged() call to IGTViewerChangeListeners.

		if (origditRow >= 0) {
			// Let editor know its data may have changed under its feet.
			// The 'IGTAbstractDataModel model' it is editing is now probably stale...
			// Or should this happen via some listener?
			// Wrapping has not occurred properly.
			
			igtViewer.updateStaleEditor(origditRow);
		}
	}
	
//	public void handleSuggestions(List<SuggestionSet> sugSets) {
//		
//	}
	
	/**
	 * We don't really have a way to show, use or manipulate the selection,
	 * which is basically just a time period. So we ignore this.
	 * On the other hand, we (or rather the IGTViewer) 
	 * do set the selection to the time period of the
	 * active annotation, when we set it.
	 */
	@Override // AbstractViewer
	public void updateSelection() {
		// stub

	}

	/**
	 * Here we get notified when someone changes the active annotation.
	 * We are a listener because <code>ViewerManager2.connect(AbstractViewer)</code>
	 * makes us one.
	 */
	@Override // AbstractViewer
	public void updateActiveAnnotation() {
		igtViewer.updateActiveAnnotation(getActiveAnnotation());
	}

	@Override // AbstractViewer
	public void updateLocale() {
		if (interButton != null) {
			interButton.setText(ElanLocale.getString("InterlinearEditor.Button.Interlinearize"));
			recursiveCB.setText(ElanLocale.getString("InterlinearEditor.CheckBox.Recursive"));
		}
	}

	@Override // AbstractViewer
	public void preferencesChanged() {
		setPreferredFontAndColorSettings();
		
		repaint();
	}
	
	/**
	 * Updates the location and sizes of the components.
	 */
	@Override // ComponentListener
	public void componentResized(ComponentEvent e) {
		if (e.getSource() == playerControlPanel) {
			int width = playerControlPanel.getWidth();
			igtViewer.setHeaderWidth(width + buttonSplitPane.getDividerSize() / 2);
		} else if (e.getSource() == this) {	
			
			int w = e.getComponent().getWidth();
			int h = e.getComponent().getHeight();
			
			buttonSplitPane.setBounds(0, 0, w, bpHeight);
			igtViewer.setBounds(0, bpHeight, w, h - bpHeight);
			//igtViewer.recalculateAllRows(); // now done instead in the viewer's componentResized().
	        validate(); // this seems needed:
	        // It should be invoked when this container's subcomponents are modified
	        // (...or layout-related information changed) after the container has
	        // been displayed.	
		}
	}

	@Override // ComponentListener
	public void componentMoved(ComponentEvent e) {
		// stub
	}

	/**
	 * Repaints.
	 */
	@Override // ComponentListener
	public void componentShown(ComponentEvent e) {
		// stub		
	}

	@Override // ComponentListener
	public void componentHidden(ComponentEvent e) {
		// stub	
	}

	@Override // ActionListener
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == interButton) {
			// first check if there is an active annotation
			Annotation activeAnn = getActiveAnnotation();
			TierImpl activeSourceTier = null; // the tier the active annotation is on, potential source tier
			AnalyzerConfig ac = activeConfig;
			// the following list will be filled with tier configurations
			List<AnalyzerConfig> activeConfList = new ArrayList<AnalyzerConfig>();
			
			if (activeAnn != null) {
				activeSourceTier = (TierImpl) activeAnn.getTier();
				String activeTypeName = activeSourceTier.getLinguisticType().getLinguisticTypeName();
				final List<AnalyzerConfig> configurations = manager.getTextAnalyzerContext().getConfigurations();
				
				for (AnalyzerConfig acf : configurations) {
					if (acf.isTypeMode()) {
						if (acf.getSource().equals(activeTypeName)) {
							// add all tier configurations of this type? or just of the active tier?
							activeConfList.addAll(acf.getTierConfigurations());
						}
					} else {
						// one tier configuration
						if (acf.getSource().equals(activeSourceTier.getName())) {
							activeConfList.add(acf);
						}
					}
				}
			} else {
				// start with active configuration or just the first configuration in the list
				if (ac == null) {
					final List<AnalyzerConfig> configurations = manager.getTextAnalyzerContext().getConfigurations();
					if (configurations.size() > 0) {
						ac = configurations.get(0);
						activeConfList.addAll(ac.getTierConfigurations());
					}
				} else {
					activeConfList.addAll(ac.getTierConfigurations());
				}
			}			

			if (/*ac == null*/activeConfList.isEmpty()) {
				JOptionPane.showMessageDialog(this,
						ElanLocale.getString("InterlinearEditor.Warning.NoConfig.Message"),
						ElanLocale.getString("InterlinearEditor.Warning.NoConfig.Title"),
					    JOptionPane.WARNING_MESSAGE);
				return;
			}
			
			manager.getTextAnalyzerContext().interLinearizeActionStarting();
			// it is best to follow the order of the annotations in the viewer
			if (activeAnn != null) {
				// start with this one
				int startRow = igtViewer.getRowIndexForTierAndTime(activeSourceTier.getName(), activeAnn.getBeginTimeBoundary());
				for (int row = startRow; row < igtViewer.getRowCount(); row++) {
					long[] timeInterval = igtViewer.getTimeIntervalForRow(row);
					if (timeInterval == null) {
						continue;//break
					}
					
					if (row == startRow) {
						List<AbstractAnnotation> annotations = activeSourceTier.getAnnotations();
						int annIndex = annotations.indexOf(activeAnn);
						for (int i = annIndex; i < annotations.size(); i++) {
							Annotation a = annotations.get(i);
							
							if (a.getBeginTimeBoundary() >= timeInterval[0] && a.getEndTimeBoundary() <= timeInterval[1]) {
								Position pos = new Position(activeSourceTier.getName(),
										//pos.tierId = ac.getDestTier();
										a.getBeginTimeBoundary(),
										a.getEndTimeBoundary());
								// XXX see below
								manager.getTextAnalyzerContext().analyze(pos); 
							} else {
								break;
							}
						}
					} else {// all other rows after the "current"
						// check whether next row contains a tier that is in the source configurations
						for (AnalyzerConfig conf : activeConfList) {
							TierImpl tier = transcription.getTierWithId(conf.getSource());
							if (tier != null) {
								if (igtViewer.isTierInRow(tier.getName(), row)) {
									List<Annotation> annInInterval = tier.getOverlappingAnnotations(
											timeInterval[0], timeInterval[1]);
									for (Annotation a : annInInterval) {
										Position pos = new Position(conf.getSource(),
												//pos.tierId = ac.getDestTier();
												a.getBeginTimeBoundary(),
												a.getEndTimeBoundary());
										// XXX see below
										manager.getTextAnalyzerContext().analyze(pos);
									}
								}
							}
						}
					}
				}
			} else {
				// TODO: this loops over all annotations of all tiers in the selected configurations, without 
				// considering the ordering in the IGT viewer. So, per tier it loops from "top to bottom" and then
				// back to the top again
				for (AnalyzerConfig conf : activeConfList) {
					TierImpl tier = transcription.getTierWithId(conf.getSource());
					if (tier != null) {
						for (Annotation a : tier.getAnnotations()) {
							Position pos = new Position(conf.getSource(),// To do: check whether source or destination needs to be passed
														//pos.tierId = ac.getDestTier();
														a.getBeginTimeBoundary(),
														a.getEndTimeBoundary());
							
							/*
							 * XXX This will loop over ALL analyzers
							 * (each one corresponding to a configuration here),
							 * not just the one of which we selected the configuration here.
							 * Any analyzer that is set up for this source tier (and was load()ed
							 * successfully!) will be doing its thing.
							 * That may also mean that no analyzer does anything, and we never
							 * know about it here... 
							 */
							manager.getTextAnalyzerContext().analyze(pos); 
						} // else warn
					} // else warn
				}
			}
			manager.getTextAnalyzerContext().interLinearizeActionCompleted();
		
		} else if(e.getSource() == recursiveCB) {
			manager.getTextAnalyzerContext().setRecursive(recursiveCB.isSelected());
		}
	}

}
