package mpi.eudico.client.annotator.interlinear.edit;

/**
 * Implements a JComponent that displays a single Suggestion.
 */
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

import javax.swing.DebugGraphics;
import javax.swing.JComponent;
import javax.swing.RepaintManager;

import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionViewerModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTBlockRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTNodeRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTViewerRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.SuggestionFlowLayout;

@SuppressWarnings("serial")
public class SuggestionComponent extends JComponent
		implements SuggestionFlowLayout.Location, MouseMotionListener {
	private int sugIndex;
	private IGTSuggestionModel sugModel;
	private IGTViewerRenderInfo viewerRenderInfo;
	private IGTSuggestionViewerModel svModel;
	private int fragNr;
	private final static int NO_FRAG = -99999;
	
	public SuggestionComponent(int sugIndex, IGTViewerRenderInfo viewerRenderInfo, IGTSuggestionViewerModel svModel) {
		this.sugIndex = sugIndex;
		this.viewerRenderInfo = viewerRenderInfo;
		this.svModel = svModel;
		this.sugModel = svModel.getRowData(sugIndex);
		this.fragNr = NO_FRAG;
		sugModel.showTierLabels(sugIndex == 0);

		if (false) {
			RepaintManager repaintManager = RepaintManager.currentManager(this);
			repaintManager.setDoubleBufferingEnabled(false);
			setDebugGraphicsOptions(DebugGraphics.BUFFERED_OPTION | DebugGraphics.FLASH_OPTION
			        | DebugGraphics.LOG_OPTION);
			setOpaque(false);	 // TESTING: where is all the flashing coming from??
		}
		//setDebugGraphicsOptions(DebugGraphics.FLASH_OPTION);
		// set silly default size
		this.sugModel.getRenderInfo().height = 75;
		this.sugModel.getRenderInfo().width = 120;
		
		// ToolTips interfere with the MouseListener on the Container^2
		// (the SuggestionWindow). mousePressed() and mouseClicked() don't
		// get called any more when it occurs inside this component...
		// For a workaround, see workAroundToolTipBug().
		String label = sugModel.getSuggestionSet().getLabel();
		if (label != null) {
			setToolTipText(label);
		}
		
		addHierarchyListener(new HierarchyListener() {
			@Override
			public void hierarchyChanged(HierarchyEvent e) {
				if ((e.getChangeFlags() & HierarchyEvent.PARENT_CHANGED) != 0 &&
						e.getChanged() == SuggestionComponent.this) {
	        		calculateLocationAndSize();
	        		removeHierarchyListener(this);
				}
			}
		});
		addMouseMotionListener(this);
	}
	
	/**
	 * Calculates the size and location of the items in the model.
	 * This needs to be delayed until this component is actually add()ed
	 * to a parent, because it uses getGraphics().
	 * 
	 * @param sugModel the suggestion model
	 */
	private void calculateLocationAndSize() {
		Graphics g = getGraphics();
		if (g == null) {
			return;	// leave size (if calculated before) alone
		}

		int h, w;

		final int tierRowCount = sugModel.getRowCount();

		if (tierRowCount > 0) {
			IGTTier rootTier = sugModel.getRootRow();
			
			// Calculate horizontal layout
			w = IGTCalculator.calculateTierAndAnnotationsLAS(g, viewerRenderInfo, 
					rootTier, true);
			
			final IGTBlockRenderInfo sugRenderInfo = sugModel.getRenderInfo();
			sugRenderInfo.width = w;

			// If the tier labels are included in this one, reflect it in the width
			if (sugModel.showTierLabels()) {
				sugRenderInfo.width += viewerRenderInfo.headerWidth;
			}

			/* The Y calculation uses the "natural" height of each line, as determined by
			 * the font size. It includes the ROOT tier which takes up space but is not
			 * actually drawn. In its place is the header, of height 
			 * svModel.getRenderInfo().headerHeight +1 (for the line of the rectangle).
			 * By manipulating the starting Y, we can position the first real tier
			 * just right at the cost of having a silly Y position for the ROOT tier.
			 * 
			 * Additionally, we could set .headerHeight to a value related to the
			 * font size used to draw the header text. Since that is global for the viewer,
			 * all suggestions should use the same font for that (which is reasonable).
			 */
			h = svModel.getRenderInfo().headerHeight + 1 -
				viewerRenderInfo.getHeightForTier(g, rootTier.getTierName());
			h += viewerRenderInfo.vertLineMargin;
			h += IGTCalculator.calculateTierYPositionRecursive(g, viewerRenderInfo, rootTier, h);
			
			h += viewerRenderInfo.vertLineMargin;
			sugRenderInfo.height = h;	
		}
	}

	public void setFragNr(int fragNr) {
		if (this.fragNr != fragNr) {
			this.fragNr = fragNr;
			repaint();
		}
	}
	
	public int getFragNr() {
		return fragNr;
	}
	
	@Override
	public void paintComponent(Graphics g) {
		if (fragNr > NO_FRAG) {
			int hash = getHashOfFrag(fragNr);
			// Use the hash to fill in 6 of the the lower 7 bits of each colour component,
			// keeping the high bit on and the low bit off, so that we have a fairly light colour
			// which results in better readability of the black text.
			// 0b1xxxxxx0
			int colour = (int)(((long)hash * 31) % 0x03FFFF);
			int cr = 0x80 | ((colour >> 11) & 0x7E);
			int cg = 0x80 | ((colour >>  5) & 0x7E);
			int cb = 0x80 | ((colour <<  1) & 0x7E);
			Color savedColor = svModel.renderInfo.blockBackGround;
			svModel.renderInfo.blockBackGround = new Color(cr, cg, cb); // HORRIBLE HACK!
	
			IGTSuggestionRenderer.renderOneSuggestion(g, sugIndex, sugModel.showTierLabels(),
					viewerRenderInfo, svModel);
			
			// Draw a rectangle around the part that determines the colour.
			Rectangle rect = getRectOfFrag(fragNr);
			if (rect != null) {
				int height = sugModel.getRenderInfo().height;
				
				g.setColor(Color.BLUE /*new Color(cr ^ 0x80, cg ^ 0x80, cb ^ 0x80)*/);
				g.drawRect(rect.x - 2, rect.y - 2, rect.width - 1 + 4, height - rect.y - 1); 
			}
			svModel.renderInfo.blockBackGround = savedColor;
		} else {
			IGTSuggestionRenderer.renderOneSuggestion(g, sugIndex, sugModel.showTierLabels(),
					viewerRenderInfo, svModel);
		}
	}

	/**
	 * Get the IGTAnnotation that represents the given fragment number.
	 * Negative numbers count from the right:
	 * -1 is the rightmost fragment.
	 * 
	 * @param fragNr in domain [-size, size)
	 * @return the IGTAnnotation, or null if the fragNr is not in the domain.
	 */
	private IGTAnnotation getFrag(int fragNr) {
		IGTSuggestionModel sugg = svModel.getRowData(sugIndex);
		IGTTier firstTier = sugg.getRowData(1);
		final ArrayList<IGTAnnotation> annotations = firstTier.getAnnotations();
		
		final int size = annotations.size();
		if (fragNr < 0) {
			fragNr = fragNr + size;
		}
		if (fragNr >= 0 && fragNr < size) {
			return annotations.get(fragNr);
		}
		
		return null;
	}
	
	/**
	 * @return The hash of the given fragment number.
	 * 
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public int getHashOfFrag(int fragNr) {
		IGTAnnotation firstPart = getFrag(fragNr);
		if (firstPart != null) {
			return firstPart.hashCodeOfText();
		}
		
		return 0;
	}
	
	/**
	 * @param fragNr
	 * @return the Rectangle that the given fragment number represents.
	 */
	public Rectangle getRectOfFrag(int fragNr) {
		IGTAnnotation firstPart = getFrag(fragNr);
		if (firstPart != null) {
			IGTNodeRenderInfo ri = firstPart.getRenderInfo();
			int offx = sugModel.showTierLabels() ? viewerRenderInfo.headerWidth : 0;
			
			return new Rectangle(offx + ri.x, ri.y, ri.calcWidth, ri.height);
		}
		
		return null;
	}
	
	/**
	 * Given an X coordinate, return which in which fragment of the suggestion this is.
	 * <p>
	 * If <code>fromRight</code> is true, count the fragments from the right,
	 * where the rightmost fragment is number -1.
	 * 
	 * Otherwise, count from the left and the leftmost fragment is number 0.
	 * 
	 * If no fragment falls at the given location, return NO_FRAG.
	 * <p>
	 * This makes the range [-size, size) U { NO_FRAG } where size is the total number of fragments.
	 */
	public int getFragOfX(int x, boolean fromRight) {
		IGTSuggestionModel sugg = svModel.getRowData(sugIndex);
		IGTTier firstTier = sugg.getRowData(1);
		final ArrayList<IGTAnnotation> annotations = firstTier.getAnnotations();
		int offx = sugModel.showTierLabels() ? viewerRenderInfo.headerWidth : 0;
		x -= offx;
	
		int fragNr = -1;
		for (IGTAnnotation ann : annotations) {
			IGTNodeRenderInfo ri = ann.getRenderInfo();
			if (ri.x > x) {
				break;
			}
			fragNr++;
		}
		
		if (fragNr < 0) {
			fragNr = NO_FRAG;
		} else if (fromRight) {
			fragNr = fragNr - annotations.size();
		}
		
		return fragNr;
	}
	
	@Override
	public Dimension getPreferredSize() {
		final IGTBlockRenderInfo renderInfo = sugModel.getRenderInfo();

		return new Dimension(renderInfo.width, renderInfo.height);
	}
	
	/**
	 * This suggestion knows its index in the list of suggestions.
	 * Return that number.
	 */
	public int getIndex() {
		return sugIndex;
	}

	/**
	 * Callback from the SuggestionFlowLayout to notify the component of its
	 * location in the layout.
	 */
	@Override // SuggestionFlowLayout.Location
	public void setGrid(int col, int row) {
		final IGTBlockRenderInfo sugRenderInfo = sugModel.getRenderInfo();

		//System.err.printf("SuggestionComponent: setGrid row=%d col=%d => ", row, col);
		if (col == 0) {
			if (!sugModel.showTierLabels()) {
				sugRenderInfo.width += viewerRenderInfo.headerWidth;
				sugModel.showTierLabels(true);
			}
		} else {
			if (sugModel.showTierLabels()) {
				sugRenderInfo.width -= viewerRenderInfo.headerWidth;
				sugModel.showTierLabels(false);
			}
		}
		//System.err.printf("%d x %d\n", renderInfo.width, renderInfo.height);
		setSize(sugRenderInfo.width, sugRenderInfo.height);
	}

	/**
	 * Get the preferred size of this component, given its location
	 * in the layout.
	 */
	@Override // SuggestionFlowLayout.Location
	public Dimension getPreferredSize(int col, int row) {
		final IGTBlockRenderInfo sugRenderInfo = sugModel.getRenderInfo();
		int width = sugRenderInfo.width;
		
		//System.err.printf("SuggestionComponent: getPreferredSize row=%d col=%d => ", row, col);
		if (col == 0) {
			if (!sugModel.showTierLabels()) {
				width += viewerRenderInfo.headerWidth;
			}
		} else {
			if (sugModel.showTierLabels()) {
				width -= viewerRenderInfo.headerWidth;
			}
		}

		//System.err.printf("%d x %d\n", width, renderInfo.height);
		return new Dimension(width, sugRenderInfo.height);
	}
	
	@Override
	public void setPreferredSize(Dimension dim) {
		// do not call super.setPreferredSize(dim);
		// but this isn't called anyway, so unneeded.
	}

	/**
	 * Shorthand helper interface name that implements both 
	 * MouseListener and MouseMotionListener.
	 * <p>
	 * Unfortunately the implementer needs to implement THIS interface;
	 * just implementing the two Listeners is not sufficient.
	 */
	public interface MouseAndMotionListener extends MouseListener, MouseMotionListener {};  

	/**
	 * This is needed because a ToolTip installs a MouseListener and
	 * therefore eats the mouse events that our parent containers may
	 * want to see (in particular the SuggestionWindow).
	 * So we work around this by installing extra mouse listeners.
	 */
	public void workAroundToolTipBug(MouseAndMotionListener mml) {
		addMouseListener(mml);
		addMouseMotionListener(mml);
	}
	
	public void removeWorkAroundToolTipBug(MouseAndMotionListener mml) {
		removeMouseListener(mml);
		removeMouseMotionListener(mml);
	}

	@Override
	public void mouseDragged(MouseEvent e) {
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		if (e.isShiftDown() || e.isAltDown()) {
			int x = e.getX();
			int frag = getFragOfX(x, e.isAltDown());
			
			//System.err.printf("SuggestionComponent:mouseMoved: x = %d, frag = %d\n", x, frag);
			if (frag != this.fragNr) {
				// Change the fragment number on all suggestions in
				// the SuggestionWindow, which happen to be our siblings (and us).
				// Probably, this should be done in a cleaner manner, since this method
				// assumes all suggestions are in the same container parent.
				Container parent = getParent();

				for (Component sibling : parent.getComponents()) {
					if (sibling instanceof SuggestionComponent) {
						SuggestionComponent sc = (SuggestionComponent)sibling;
						sc.setFragNr(frag);
					}
				}
			}
		}
	}

	/** for setting a breakpoint */
//	@Override
//	public void update(Graphics g) {
//		super.update(g);
//	}
}
