package mpi.eudico.client.annotator.interlinear.edit.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import mpi.eudico.client.annotator.ViewerManager2;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.interlinear.edit.TextAnalyzerHostContext;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import nl.mpi.lexan.analyzers.helpers.Position;

/**
 * Temporary class for deleting an annotation. 
 * Should be undoable.
 * 
 * @author Han Sloetjes
 *
 */
@SuppressWarnings("serial")
public class IGTDeleteAction extends IGTEditAction {
	/** The mediator connector to use for starting the interlinearization action 
	 * In case of deletion the connected analyzer might need to know of the removal? */
	protected TextAnalyzerHostContext hostContext;
	
	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param hostContext the mediator via which the connected analyzer can be informed of deletion
	 */
	public IGTDeleteAction(IGTAnnotation igtAnnotation, TextAnalyzerHostContext hostContext) {
		this(igtAnnotation, hostContext, null, null);
	}

	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param hostContext the mediator via which the connected analyzer can be informed of deletion
	 * @param label the text for the action 
	 */	
	public IGTDeleteAction(IGTAnnotation igtAnnotation, TextAnalyzerHostContext hostContext,
			String label) {
		this(igtAnnotation, hostContext, label, null);
	}
	
	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param hostContext the mediator via which the connected analyzer can be informed of deletion
	 * @param label the text for the action
	 * @param icon an icon for the action 
	 */
	public IGTDeleteAction(IGTAnnotation igtAnnotation, TextAnalyzerHostContext hostContext, String label, 
			Icon icon) {
		super(igtAnnotation, label, icon);
		this.hostContext = hostContext;
	}

	/**
	 * 
	 */
	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (igtAnnotation != null && hostContext != null) {
			Position pos = new Position(igtAnnotation.getAnnotation().getTier().getName(),
										igtAnnotation.getAnnotation().getBeginTimeBoundary(),
										igtAnnotation.getAnnotation().getEndTimeBoundary());
			
			AbstractAnnotation aa = igtAnnotation.getAnnotation();

			if (aa != null) {
				TierImpl tier = (TierImpl) aa.getTier();
				Transcription tr = hostContext.getTranscription();
				ViewerManager2 vm = ELANCommandFactory.getViewerManager(tr);
				
				Command command = ELANCommandFactory.createCommand(tr,
						ELANCommandFactory.DELETE_ANNOTATION);
				command.execute(tier, new Object[] { vm, aa });

				// this might need changing
				hostContext.annotationDeleted(pos);
			}
		}
	}
}
