package mpi.eudico.client.annotator.interlinear.edit.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.interlinear.edit.TextAnalyzerHostContext;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AlignableAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import nl.mpi.lexan.analyzers.helpers.Position;

/**
 * Class for deleting an annotation. 
 * Is undoable because it uses an UndoableCommand to perform the work.
 * 
 * @author Olaf Seibert
 *
 */
@SuppressWarnings("serial")
public class IGTSplitAnnotationAction extends IGTEditAction {
	public static final int SPLIT = 0;
	public static final int NEW_BEFORE = 1;
	public static final int NEW_AFTER = 2;

	/**
	 * The TextAnalyzerHostContext to use for starting the interlinearization action 
	 * In case of deletion the connected analyzer might need to know of the removal?
	 */
	protected TextAnalyzerHostContext hostContext;
	/**
	 * One of SPLIT, NEW_BEFORE or NEW_AFTER.
	 */
	protected int variant;
	
	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param hostContext the mediator via which the connected analyzer can be informed of deletion
	 */
	public IGTSplitAnnotationAction(IGTAnnotation igtAnnotation,
			TextAnalyzerHostContext hostContext, int variant) {
		super(igtAnnotation);
		this.hostContext = hostContext;
		this.variant = variant;
	}

	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param hostContext the mediator via which the connected analyzer can be informed of deletion
	 * @param label the text for the action 
	 */	
	public IGTSplitAnnotationAction(IGTAnnotation igtAnnotation,
			TextAnalyzerHostContext hostContext, int variant, String label) {
		super(igtAnnotation, label);
		this.hostContext = hostContext;
		this.variant = variant;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param hostContext the mediator via which the connected analyzer can be informed of deletion
	 * @param label the text for the action
	 * @param icon an icon for the action 
	 */
	public IGTSplitAnnotationAction(IGTAnnotation igtAnnotation,
			TextAnalyzerHostContext hostContext, int variant, String label, 
			Icon icon) {
		super(igtAnnotation, label, icon);
		this.hostContext = hostContext;
		this.variant = variant;
	}

	/**
	 * Do it.
	 */
	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (igtAnnotation != null && hostContext != null) {
			AbstractAnnotation aa = igtAnnotation.getAnnotation();

			if (aa != null) {
				Transcription tr = hostContext.getTranscription();
				Position pos = new Position(aa.getTier().getName(),
											aa.getBeginTimeBoundary(), aa.getEndTimeBoundary());

				if (variant == SPLIT) {
					if (aa instanceof AlignableAnnotation) {
						Command command = ELANCommandFactory.createCommand(tr,
								ELANCommandFactory.SPLIT_ANNOTATION);
						
						Object[] args = new Object[] { aa };
						command.execute(tr, args);
					} else {
						// Error...
					}
				} else {
					TierImpl t = (TierImpl) aa.getTier();
					String cmd = variant == NEW_AFTER 
							? ELANCommandFactory.NEW_ANNOTATION_AFTER
							: ELANCommandFactory.NEW_ANNOTATION_BEFORE;
					Command command = ELANCommandFactory.createCommand(tr, cmd);
					Object[] args = new Object[] { aa };
					command.execute(t, args);
				}

				hostContext.annotationsAdded(pos);
			}
		}
	}
}
